/* $Id: BoardHeuristic.java,v 1.3 2007/11/28 17:49:42 ekstrand Exp $ */

package csci1902.hw6.ai;

import csci1902.hw6.game.Board;
import csci1902.hw6.game.PlayerID;

/**
 * Interface for board heuristic functions.
 * 
 * <p>It is important for heuristic implementations to not depend on any state
 * kept between calls to {@link #computeScore(Board, PlayerID)}.  There are no
 * guarantees as to how long a heuristic will be kept around or how many times
 * the same heuristic object will be reused.  Think of them like comparators --
 * they only exist to have their method called an arbitrary number of times in
 * an arbitrary sequence.
 * 
 * @author Michael Ekstrand <ekstrand@cs.umn.edu>
 */
public interface BoardHeuristic {
    /**
     * Compute the heuristic score for a particular player on a board.
     * @param board The board to score.
     * @param player The player whose score is desired.
     * @return The heuristic score of the board.
     */
    public double computeScore(Board board, PlayerID player);
}
