package csci1902.hw6.ai;
import java.util.*;

import csci1902.hw6.game.Board;
import csci1902.hw6.game.CellPosition;
import csci1902.hw6.game.PlayerID;
/**
* Class representing a node in the game tree.
*/
class TreeNode {
private Map<Board,TreeNode> cachedNodes;
private Board board;
private PlayerID player;
/**
* Construct a new root tree node.
* @param player The first player of the game.
* @param rows The number of rows in the game board.
* @param columns The number of columns in the game board.
* @param goal The goal of the game board.
*/
public TreeNode(PlayerID player, int rows, int columns, int goal) {
	this(player, new Board(rows, columns, goal), new HashMap<Board, TreeNode>());
}
/**
* Construct a new child tree node.
* @param player The player at this node.
* @param board The current board at this node.
* @param cache The memoization cache at this node.
*/
private TreeNode(PlayerID player, Board board, Map<Board,TreeNode> cache) {
	this.player = player;
	this.board = board;
	cachedNodes = cache;
}
/**
* Get a child node of this tree node.
* @param move The move made to reach the child.
* @return The child node obtained by moving {@code move} from this node's
* state.
*/
public TreeNode getChild(CellPosition move) {
	
	Board newBoard = new Board(board, player, move);
	if (cachedNodes.containsKey(newBoard)){
		return cachedNodes.get(newBoard);
	}
	TreeNode newNode = new TreeNode(player.otherPlayer(), newBoard, cachedNodes);
	cachedNodes.put(newBoard, newNode);
	return newNode;	
}
/**
* Retrieve the game board at this node.
* @return The game board containing the state at this node.
*/
public Board getBoard() {
	return board;
}
/**
* Retrieve the player ID of this node.
* @return The player who plays from this node.
14
*/
public PlayerID getPlayer() {
	return player;
}
/**
* Query the leaf status of the node.
* @return {@code true} if the node is a leaf node (the game is finished).
*/
public boolean isLeaf() {
	return board.isFinished();
}
/**
* Query the available moves from this node.
* @return the collection of available moves in the board.
*/
public Collection<CellPosition> getSuccessorMoves() {
	if(board.isFinished()){
		return null;
	}
	return board.getAvailableMoves();
}
/**
* Return the internal cache. Only for testing purposes.
* @return The internal memoization cache.
*/
Map<Board,TreeNode> getCache() {
	return cachedNodes;
}
}