/* $Id: TreeNodeTest.java,v 1.3 2007/11/28 17:49:42 ekstrand Exp $ */

package csci1902.hw6.ai;

import csci1902.hw6.game.Board;
import csci1902.hw6.game.PlayerID;
import junit.framework.TestCase;

/**
 * Test cases for the TreeNode class.
 * @author Michael Ekstrand <ekstrand@cs.umn.edu>
 *
 */
public class TreeNodeTest extends TestCase {
    private TreeNode node;
    private Board base;
    
    /**
     * Set up the initial base board and node.
     */
    @Override
    protected void setUp() {
        node = new TreeNode(PlayerID.X, 3, 6, 3);
        base = new Board(3,6,3);
    }
    
    /**
     * Test the initial state of the node.
     */
    public void testInitialState() {
        assertNotNull(node.getBoard());
        assertEquals(base, node.getBoard());
        assertNotNull(node.getPlayer());
        assertEquals(PlayerID.X, node.getPlayer());
        assertFalse(node.isLeaf());
        assertNotNull(node.getCache());
        assertTrue(node.getCache().isEmpty());
    }
    
    /**
     * Simple test for child construction.
     */
    public void testGoChild() {
        TreeNode child = node.getChild(base.makeCellPosition(0, 0));
        Board b = new Board(base, PlayerID.X, base.makeCellPosition(0, 0));
        assertEquals(b, child.getBoard());
        assertFalse(node.getCache().isEmpty());
        assertTrue(node.getCache().containsKey(b));
        assertSame(child, node.getCache().get(b));
        assertSame(node.getCache(), child.getCache());
        assertEquals(PlayerID.O, child.getPlayer());
    }
    
    /**
     * Test the memoization.
     */
    public void testMemoize() {
        TreeNode k11 = node.getChild(base.makeCellPosition(0,0));
        TreeNode k12 = k11.getChild(base.makeCellPosition(1,0));
        TreeNode k13 = k12.getChild(base.makeCellPosition(0,1));
        
        TreeNode k21 = node.getChild(base.makeCellPosition(0,1));
        TreeNode k22 = k21.getChild(base.makeCellPosition(1,0));
        TreeNode k23 = k22.getChild(base.makeCellPosition(0,0));
        
        assertSame(k13, k23);
    }
    
    /**
     * Test the successor nodes.
     */
    public void testSuccessors() {
        assertEquals(2, node.getSuccessorMoves().size());
        assertEquals(base.getAvailableMoves(), node.getSuccessorMoves());
        
        TreeNode kid = node.getChild(base.makeCellPosition(3,5));
        assertEquals(kid.getBoard().getAvailableMoves(), kid.getSuccessorMoves());
    }
}
