/* $Id: CellPosition.java,v 1.7 2007/11/28 17:49:42 ekstrand Exp $ */

package csci1902.hw6.game;


/**
 * Class representing a cell position or move in circular tic-tac-toe.
 * @author Michael Ekstrand <ekstrand@cs.umn.edu>
 *
 */
public class CellPosition implements Comparable<CellPosition> {
    /** The cell's row */
    public final int row;
    /** The cell's column */
    public final int column;
    
    /**
     * Constructs a new cell position.
     *
     * Non-engine components (e.g. user interface and AI engines) cannot
     * create cell position objects directly.  Rather, they must get one from
     * a board with {@link Board#makeCellPosition(int, int)}.
     * @param config The board configuration against which the cell is
     * normalized.
     * @param row The row of the position.
     * @param column The column of the position.
     */
    CellPosition(Board.Config config, int row, int column) {
        if (row < 0 || row > config.rowCount)
            throw new IndexOutOfBoundsException("Invalid row " + row);
        
        this.row = row;
        
        while (column < 0)
            column += config.columnCount;
        this.column = column % config.columnCount;
    }
    
    @Override
    public boolean equals(Object other) {
        if (other instanceof CellPosition) {
            CellPosition pos = (CellPosition) other;
            return pos.row == row && pos.column == column;
        } else {
            return false;
        }
    }
    
    @Override
    public String toString() {
        return String.format("(%d,%d)", row, column);
    }

    /**
     * Allow cell positions to be compared for mapping purposes.
     */
    public int compareTo(CellPosition other) {
        int diff = row - other.row;
        if (diff != 0)
            return diff;
        else
            return column - other.column;
    }
}
